@@ -6,7 +6,6 @@ import math
 import time
 from typing import Any, Final, Optional
 
-from pyplumio.helpers.parameter import Parameter
 from pyplumio.helpers.typing import NumericType, SensorCallbackType
 
 TOLERANCE: Final = 0.1
@@ -14,9 +13,6 @@ TOLERANCE: Final = 0.1
 
 def _significantly_changed(old_value, new_value) -> bool:
     """Check if value is significantly changed."""
-    if old_value is None or (isinstance(old_value, Parameter) and old_value.changed):
-        return True
-
     if isinstance(old_value, (int, float)) and isinstance(new_value, (int, float)):
         return not math.isclose(old_value, new_value, abs_tol=TOLERANCE)
 
