@@ -1,7 +1,8 @@
 """Contains reader and writer classes."""
 from __future__ import annotations
 
-from asyncio import StreamReader, StreamWriter
+from asyncio import IncompleteReadError, StreamReader, StreamWriter
+import logging
 from typing import Final, Optional, Tuple
 
 from pyplumio import util
@@ -17,6 +18,8 @@ WRITER_TIMEOUT: Final = 10
 MIN_FRAME_LENGTH: Final = 10
 MAX_FRAME_LENGTH: Final = 1000
 
+_LOGGER = logging.getLogger(__name__)
+
 
 class FrameWriter:
     """Represents frame writer."""
@@ -33,6 +36,7 @@ class FrameWriter:
         wait for buffer to drain."""
         self._writer.write(frame.bytes)
         await self._writer.drain()
+        _LOGGER.debug("Sent frame: %s", frame)
 
     @timeout(WRITER_TIMEOUT)
     async def close(self) -> None:
@@ -52,25 +56,26 @@ class FrameReader:
 
     async def _read_header(self) -> Tuple[bytes, int, int, int, int, int]:
         """Locate and read frame header."""
-        while True:
-            header = await self._reader.read(1)
-            if header and header[0] == FRAME_START:
-                break
+        while buffer := await self._reader.read(1):
+            if FRAME_START not in buffer:
+                continue
 
-        header += await self._reader.read(HEADER_SIZE - 1)
-        if len(header) < HEADER_SIZE:
-            raise ReadError(f"Header can't be less than {HEADER_SIZE} bytes")
+            buffer += await self._reader.read(HEADER_SIZE - 1)
+            if len(buffer) < HEADER_SIZE:
+                raise ReadError(f"Header can't be less than {HEADER_SIZE} bytes")
 
-        [
-            _,
-            length,
-            recipient,
-            sender,
-            sender_type,
-            econet_version,
-        ] = util.unpack_header(header)
+            [
+                _,
+                length,
+                recipient,
+                sender,
+                sender_type,
+                econet_version,
+            ] = util.unpack_header(buffer)
 
-        return header, length, recipient, sender, sender_type, econet_version
+            return buffer, length, recipient, sender, sender_type, econet_version
+
+        raise OSError("No data can be read, RS485 connection broken")
 
     @timeout(READER_TIMEOUT)
     async def read(self) -> Optional[Frame]:
@@ -84,28 +89,31 @@ class FrameReader:
             econet_version,
         ) = await self._read_header()
 
-        if recipient in (ADDR_ECONET, ADDR_BROADCAST):
-            # Destination address is econet or broadcast.
-            payload = await self._reader.read(length - HEADER_SIZE)
-            payload_size = len(payload)
+        if recipient not in (ADDR_ECONET, ADDR_BROADCAST):
+            return None
 
-            if (
-                length > MAX_FRAME_LENGTH
-                or length < MIN_FRAME_LENGTH
-                or (payload_size + HEADER_SIZE) != length
-            ):
-                raise ReadError(f"Unexpected frame size ({payload_size})")
+        if length > MAX_FRAME_LENGTH or length < MIN_FRAME_LENGTH:
+            raise ReadError(f"Unexpected frame length ({length})")
 
-            if payload[-2] != util.crc(header + payload[:-2]):
-                raise ChecksumError(f"Incorrect frame checksum ({payload[-2]})")
+        try:
+            payload = await self._reader.readexactly(length - HEADER_SIZE)
+        except IncompleteReadError as e:
+            raise ReadError(
+                "Got an incomplete frame while trying to read "
+                + f"'{length - HEADER_SIZE}' bytes"
+            ) from e
 
-            return factory(
-                get_frame_handler(frame_type=payload[0]),
-                recipient=recipient,
-                message=payload[1:-2],
-                sender=sender,
-                sender_type=sender_type,
-                econet_version=econet_version,
-            )
+        if payload[-2] != util.crc(header + payload[:-2]):
+            raise ChecksumError(f"Incorrect frame checksum ({payload[-2]})")
 
-        return None
+        frame = factory(
+            get_frame_handler(frame_type=payload[0]),
+            recipient=recipient,
+            message=payload[1:-2],
+            sender=sender,
+            sender_type=sender_type,
+            econet_version=econet_version,
+        )
+        _LOGGER.debug("Received frame: %s", frame)
+
+        return frame
